package controllers

import (
	"ERP_project/Nats/NatsServer"
	"ERP_project/conf"
	"ERP_project/models/Account"
	"ERP_project/models/Project"
	"fmt"
	userlibs "git.baozhida.cn/ERP_libs/User"
	"git.baozhida.cn/ERP_libs/lib"
	"github.com/beego/beego/v2/adapter/orm"
	beego "github.com/beego/beego/v2/server/web"
	"math"
)

type ProjectController struct {
	beego.Controller
	User userlibs.User
}

func (c *ProjectController) Prepare() {
	c.User = *Account.User_r
}

func (c *ProjectController) Project_Finance_List() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	T_uuid := c.GetString("T_uuid")

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ProjectDao := Project.NewProject(orm.NewOrm())
	R_List, R_cnt := ProjectDao.Read_Project_Finance_List(T_uuid, page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

// 部门负责人查看的权限列表
func (c *ProjectController) Project_List() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ProjectDao := Project.NewProject(orm.NewOrm())
	R_List, R_cnt := ProjectDao.Read_Project_List("", c.User.T_uuid, page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

// 用户项目列表
func (c *ProjectController) Project_User_List() {

	// 分页参数 初始化
	page, _ := c.GetInt("page")
	if page < 1 {
		page = 1
	}
	page_z, _ := c.GetInt("page_z")
	if page_z < 1 {
		page_z = conf.Page_size
	}

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	ProjectDao := Project.NewProject(orm.NewOrm())
	R_List, R_cnt := ProjectDao.Read_Project_List(c.User.T_uuid, "", page, page_z)

	var r_jsons lib.R_JSONS
	r_jsons.Num = R_cnt
	r_jsons.Data = R_List
	r_jsons.Page = page
	r_jsons.Page_size = int(math.Ceil(float64(R_cnt) / float64(page_z)))

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: r_jsons}
	c.ServeJSON()
	return
}

func (c *ProjectController) Project_Get() {

	// 查询
	T_id, _ := c.GetInt("T_id")

	o := orm.NewOrm()
	ProjectDao := Project.NewProject(o)
	//ProjectDetailDao := Project.NewProjectDetail(o)
	project, err := ProjectDao.Read_Project_ById(T_id)
	if err != nil {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "查询失败!"}
		c.ServeJSON()
		return
	}

	//detailList := ProjectDetailDao.Read_ProjectDetail_List(project.Id)

	userList, _ := NatsServer.Read_User_List_All()
	Account.Read_User_All_Map(userList)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: Project.ProjectToProject_Detail(project)}
	c.ServeJSON()
	return
}

func (c *ProjectController) Project_Add() {

	T_name := c.GetString("T_name")
	T_start_date := c.GetString("T_start_date")
	T_end_date := c.GetString("T_end_date")
	T_set_up_date := c.GetString("T_set_up_date")
	T_description := c.GetString("T_description")
	T_planning_cycle, _ := c.GetFloat("T_planning_cycle")
	T_reality_cycle, _ := c.GetFloat("T_reality_cycle")
	T_bonus, _ := c.GetFloat("T_bonus")
	T_Perf, _ := c.GetFloat("T_Perf")
	T_remark := c.GetString("T_remark")
	T_detail := c.GetString("T_detail")
	T_approver := c.GetString("T_approver")

	if len(T_name) == 0 {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "项目名称不能为空！"}
		c.ServeJSON()
		return
	}

	var_ := Project.Project{
		T_uid:            c.User.T_uuid,
		T_name:           T_name,
		T_start_date:     T_start_date,
		T_end_date:       T_end_date,
		T_set_up_date:    T_set_up_date,
		T_description:    T_description,
		T_planning_cycle: float32(T_planning_cycle),
		T_reality_cycle:  float32(T_reality_cycle),
		T_bonus:          float32(T_bonus),
		T_Perf:           float32(T_Perf),
		T_remark:         T_remark,
		T_detail:         T_detail,
		T_approver:       T_approver,
		T_State:          Project.WaitAudit,
	}

	o := orm.NewOrm()
	//o.Begin()
	ProjectDao := Project.NewProject(o)
	//ProjectDetailDao := Project.NewProjectDetail(o)

	id, err := ProjectDao.Add_Project(var_)
	if err != nil {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
		c.ServeJSON()
		return
	}

	//detailList := lib.SplitString(T_detail, "|")
	//for _, v := range detailList {
	//	T_function := strings.Split(v, ",")[0]
	//	T_planned_time := strings.Split(v, ",")[1]
	//	T_finish := strings.Split(v, ",")[2]
	//	temp, _ := strconv.ParseFloat(T_planned_time, 64)
	//	T_planned_time64, _ := decimal.NewFromFloat(temp).Round(2).Float64()
	//
	//	detail := Project.ProjectDetail{
	//		T_project_id:   int(id),
	//		T_function:     T_function,
	//		T_planned_time: float32(T_planned_time64),
	//		T_finish:       T_finish,
	//	}
	//	_, err = ProjectDetailDao.Add_ProjectDetail(detail)
	//	if err != nil {
	//		o.Rollback()
	//		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
	//		c.ServeJSON()
	//		return
	//	}
	//
	//}

	//o.Commit()
	NatsServer.AddNews(T_approver, fmt.Sprintf("您有一条【%s】的立项申请待审批", c.User.T_name), conf.ProjectApprovalUrl)

	NatsServer.AddUserLogs(c.User.T_uuid, "项目", "添加", var_)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: id}
	c.ServeJSON()
	return
}

func (c *ProjectController) Project_Approval() {

	T_id, _ := c.GetInt("T_id")
	T_state, _ := c.GetInt("T_state")

	o := orm.NewOrm()
	ProjectDao := Project.NewProject(o)

	project, err := ProjectDao.Read_Project_ById(T_id)
	if err != nil {
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "T_id Err！"}
		c.ServeJSON()
		return
	}
	// 1 待审核  2 进行中  3 已完成 4 已发绩效
	project.T_State = T_state
	_, err = ProjectDao.Update_Project(project, "T_State")
	if err != nil {
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
		c.ServeJSON()
		return
	}
	if T_state == Project.HaveInHand {
		NatsServer.AddNews(project.T_uid, fmt.Sprintf("您提交的项目(%s)【进行中】", project.T_name), conf.MyProjectUrl)
	}
	if T_state == Project.Completed {
		NatsServer.AddNews(project.T_uid, fmt.Sprintf("您提交的项目(%s)【已完成】", project.T_name), conf.MyProjectUrl)
	}
	if T_state == Project.PayPerf {
		NatsServer.AddNews(project.T_uid, fmt.Sprintf("您提交的项目(%s)【已发绩效】", project.T_name), conf.MyProjectUrl)
	}
	NatsServer.AddUserLogs(c.User.T_uuid, "项目", "审核", project)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: T_id}
	c.ServeJSON()
	return
}
func (c *ProjectController) Project_Edit() {
	T_id, _ := c.GetInt("T_id")
	T_name := c.GetString("T_name")
	T_end_date := c.GetString("T_end_date")
	T_set_up_date := c.GetString("T_set_up_date")
	T_description := c.GetString("T_description")
	T_planning_cycle, _ := c.GetFloat("T_planning_cycle")
	T_reality_cycle, _ := c.GetFloat("T_reality_cycle")
	T_bonus, _ := c.GetFloat("T_bonus")
	T_Perf, _ := c.GetFloat("T_Perf")
	T_remark := c.GetString("T_remark")
	T_detail := c.GetString("T_detail")

	o := orm.NewOrm()
	//o.Begin()
	ProjectDao := Project.NewProject(o)
	//ProjectDetailDao := Project.NewProjectDetail(o)

	project, err := ProjectDao.Read_Project_ById(T_id)
	if err != nil {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "T_id Err！"}
		c.ServeJSON()
		return
	}

	clos := []string{}
	if len(T_name) > 0 {
		project.T_name = T_name
		clos = append(clos, "T_name")
	}

	if len(T_end_date) > 0 {
		project.T_end_date = T_end_date
		clos = append(clos, "T_end_date")

	}
	if len(T_set_up_date) > 0 {
		project.T_set_up_date = T_set_up_date
		clos = append(clos, "T_set_up_date")

	}
	if len(T_description) > 0 {
		project.T_description = T_description
		clos = append(clos, "T_description")

	}
	if T_planning_cycle > 0 {
		project.T_planning_cycle = float32(T_planning_cycle)
		clos = append(clos, "T_planning_cycle")

	}
	if T_reality_cycle > 0 {
		project.T_reality_cycle = float32(T_reality_cycle)
		clos = append(clos, "T_reality_cycle")
	}
	if T_bonus > 0 {
		project.T_bonus = float32(T_bonus)
		clos = append(clos, "T_bonus")

	}
	if T_Perf > 0 {
		project.T_Perf = float32(T_Perf)
		clos = append(clos, "T_Perf")

	}
	if len(T_remark) > 0 {
		project.T_remark = T_remark
		clos = append(clos, "T_remark")
	}
	if len(T_detail) > 0 {
		project.T_detail = T_detail
		clos = append(clos, "T_detail")
	}

	// 1 待审核  2 进行中  3 已完成 4 已发绩效 项目状态为未通过，修改之后将状态更改待提交
	if project.T_State == 2 && c.User.T_uuid == project.T_uid {
		project.T_State = 1
		NatsServer.AddNews(project.T_approver, fmt.Sprintf("您有一条【%s】的立项申请待审批", c.User.T_name), conf.ProjectApprovalUrl)
	}

	_, err = ProjectDao.Update_Project(project, clos...)
	if err != nil {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
		c.ServeJSON()
		return
	}

	//if len(T_detail) > 0 {
	//	err = ProjectDetailDao.Delete_ProjectDetail_ByT_project_id(project.Id)
	//	if err != nil {
	//		o.Rollback()
	//		c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
	//		c.ServeJSON()
	//		return
	//	}
	//}
	//
	//detailList := lib.SplitString(T_detail, "|")
	//for _, v := range detailList {
	//	T_function := strings.Split(v, ",")[0]
	//	T_planned_time := strings.Split(v, ",")[1]
	//	T_finish := strings.Split(v, ",")[2]
	//	temp, _ := strconv.ParseFloat(T_planned_time, 64)
	//	T_planned_time64, _ := decimal.NewFromFloat(temp).Round(2).Float64()
	//
	//	detail := Project.ProjectDetail{
	//		T_project_id:   project.Id,
	//		T_function:     T_function,
	//		T_planned_time: float32(T_planned_time64),
	//		T_finish:       T_finish,
	//	}
	//	_, err = ProjectDetailDao.Add_ProjectDetail(detail)
	//	if err != nil {
	//		o.Rollback()
	//		c.Data["json"] = lib.JSONS{Code: 203, Msg: "添加失败"}
	//		c.ServeJSON()
	//		return
	//	}
	//
	//}
	//
	//o.Commit()

	NatsServer.AddUserLogs(c.User.T_uuid, "项目", "修改", project)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: project.Id}
	c.ServeJSON()
	return
}
func (c *ProjectController) Project_Del() {

	T_id, _ := c.GetInt("T_id")

	o := orm.NewOrm()
	//o.Begin()
	ProjectDao := Project.NewProject(o)
	//ProjectDetailDao := Project.NewProjectDetail(o)

	project, err := ProjectDao.Read_Project_ById(T_id)
	if err != nil {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "T_id Err！"}
		c.ServeJSON()
		return
	}
	// 1 待审核  2 进行中  3 已完成 4 已发绩效
	if project.T_State > 1 {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "项目进行中，禁止删除！"}
		c.ServeJSON()
		return
	}

	//err = ProjectDetailDao.Delete_ProjectDetail_ByT_project_id(project.Id)
	//if err != nil {
	//	o.Rollback()
	//	c.Data["json"] = lib.JSONS{Code: 203, Msg: "修改失败"}
	//	c.ServeJSON()
	//	return
	//}

	_, err = ProjectDao.Delete_Project(project)
	if err != nil {
		//o.Rollback()
		c.Data["json"] = lib.JSONS{Code: 202, Msg: "删除失败!"}
		c.ServeJSON()
		return
	}

	//o.Commit()

	NatsServer.AddUserLogs(c.User.T_uuid, "项目", "删除", project.Id)

	c.Data["json"] = lib.JSONS{Code: 200, Msg: "ok!", Data: project.Id}
	c.ServeJSON()
	return
}
